//
//  Channel.cpp
//  e_bash
//
//  Created by Arnaud Blanchard on 29/09/2019.
//

#include "Process.h"
#include "Channel.hpp"
#include "blc_channel.h"
#include "string"
#include <vector>
#include <forward_list>

#include <vte/vte.h>
#include <thread>
#include <iostream>

using namespace Gtk;
using namespace Glib; //signal_idle,RefPtr

using  namespace std;

vector <char*> argv_vec;


struct ChannelMenu:Gtk::Menu{
    ChannelMenu(Gtk::TreeView &tree_view);

};




struct ModelColumns : public TreeModel::ColumnRecord{
    TreeModelColumn<int> id, pid;
    TreeModelColumn<string> name, type, format, dimensions, icon;
    TreeModelColumn<Process*> process;
    
    
    ModelColumns(){
        add(id);
        add(name);
        add(type);
        add(format);
        add(dimensions);
        add(icon);
        add(pid);
        add(process);
    }
    
    void add_channel(RefPtr<TreeStore>  &tree, blc_channel *channel){
        uint32_t type_str, format_str;
        char dims_str[256];
        TreeModel::Row childrow;
        
        auto row = *(tree->append()); //* is because it is an iterator and we want the content of the ref.
        //row[id] = channel->id;
        row[name] = channel->name;
        row[type] = string(UINT32_TO_STRING(type_str, channel->type), 4);
        row[format] = string(UINT32_TO_STRING(format_str, channel->format), 4);
        channel->sprint_dims(dims_str, 256);
        row[dimensions] = dims_str;
        row[process]=nullptr;
        row[id]=channel->id;
        
        // row[icon] =  Gtk::Stock::YES.id; //dialog-information";
    }
}m_Columns;

static void refresh_channels(void* user_data){
    
    auto channel_widget = (ChannelWidget *)user_data;
    signal_idle().connect([channel_widget](){ //important to use an idle otherwise Thread problems
        channel_widget->refresh_channel_list();
        return false;
    });
}

ChannelMenu::ChannelMenu(TreeView &tree_view){
    MenuItem *item = new MenuItem("_Remove", true);
    append(*item);
    accelerate(*this);
    show_all();
    item->signal_activate().connect([&tree_view](){
        tree_view.get_selection()->selected_foreach_iter([](const TreeModel::iterator& iter){
            TreeModel::Row row = *iter;
            string name= row[m_Columns.name];
            blc_remove_channel_with_name(name.data());
        });;
    });
}


void ChannelWidget::add_process( TreeModel::Row &row){
    TreeModel::Row childrow;
    string command;
    
    Process *process = new Process(blaar_dir+"/bin/", "o_gnuplot");
    process->add_arg(((string)row[m_Columns.name]).data());
    process->run();
    
    command=blc_program_name;
    
    for (auto arg = process->arg_vector.begin()+1; arg!=process->arg_vector.end()-1; arg++){
        command+=" ";
        command+=*arg;
    }
    
    childrow = *(tree_store->append(row.children()));
    childrow[m_Columns.name]=command;
    childrow[m_Columns.icon]="utilities-terminal";
    childrow[m_Columns.process]= process;
    show_all_children();
}

void ChannelWidget::display_process( TreeModel::Row &row){
    Process *process=row[m_Columns.process];
    
    process->display_terminal();
}


ChannelWidget::ChannelWidget(){
    CellRendererPixbuf     cellRendererPixBuf;
    
    this->set_enable_search();
   // this->m
    
    blaar_dir=getenv("BLAAR_DIR");
    if (blaar_dir.empty()) EXIT_ON_ERROR("Your environment variable 'BLAAR_DIR' is not defined.");
    
    tree_store = TreeStore::create(m_Columns);
    set_model(tree_store);
    
    append_column("Name", m_Columns.name);
    append_column("Type", m_Columns.type);
    append_column("Format", m_Columns.format);
    append_column("Dimensions", m_Columns.dimensions);
    
    int numcols = append_column("image ", cellRendererPixBuf);
    get_column(numcols-1)->add_attribute(cellRendererPixBuf, "icon-name",  m_Columns.icon);
    
    refresh_channel_list();
    blc_channel_check_for_event(refresh_channels, this);
    get_selection()->set_mode(SELECTION_MULTIPLE);
    show_all_children();
    
   // set_events(Gdk::BUTTON_PRESS_MASK);
    
    signal_row_activated().connect([this](const TreeModel::Path& path, TreeViewColumn* column){
        TreeIter iter=this->tree_store->get_iter(path);
        TreeModel::Row row = *iter;
        if (tree_store->iter_depth(iter) == 0) this->add_process(row);
        else display_process(row);
    });
    
    get_selection()->signal_changed().connect([](){
        return true;
    });
    
    signal_button_release_event().connect([this](GdkEventButton const *event){
    
        if  (event->button == 3){
            ChannelMenu *m_Menu_Popup = new ChannelMenu(*this);
            m_Menu_Popup->popup_at_pointer((GdkEvent*)event);
        
        }
        return true;
    });
}

/*
void ChannelWidget::menu_add_program_item(Menu &menu, string name, string program){
    MenuItem *item = new MenuItem(name);
    
    item->signal_activate().connect([this](){
        auto refSelection = get_selection();
        if (refSelection) {
            TreeModel::iterator iter = refSelection->get_selected();
            TreeModel::Row row = *iter;
            for (TreeModel::iterator iter = refSelection-> (); ){
                
                string name= row[m_Columns.name];
                blc_remove_channel_with_name(name.data());
            }
        }
    });
    menu.append(*item);
};*/

void ChannelWidget::refresh_channel_list(){
    bool found;
    int channels_nb;
    blc_channel *channel_infos=nullptr;
    channels_nb = blc_channel_get_all_infos(&channel_infos);
    list<TreeModel::iterator> list_iters_without_channels;
    
    /* Generate a list of iter on row to be able to remove the row if the channel does not exist anymore*/
    tree_store->foreach_iter([&list_iters_without_channels, this](TreeModel::iterator const &iter){
        if (this->tree_store->iter_depth(iter)==0) list_iters_without_channels.push_back(iter); //copy iter
        return false;
    });
    
    for(blc_channel *channel=channel_infos;  channel != channel_infos+channels_nb; channel++){
        found=false;
        auto list_iter=list_iters_without_channels.begin();
        while(list_iter != list_iters_without_channels.end()){
            TreeModel::Row row = *(*list_iter);
            if (row[m_Columns.id]==channel->id){
                found=true;
                list_iter=list_iters_without_channels.erase(list_iter);
            }
            else list_iter++;
        }
        
        if (found==false)  m_Columns.add_channel(tree_store, channel);
    }
    for(auto list_iter: list_iters_without_channels){
        tree_store->erase(*list_iter);
    }
    this->show_all_children();
    free(channel_infos);
}
