//
//  Process.cpp
//  e_bash
//
//  Created by Arnaud Blanchard on 30/08/2017.
//
//
#include "Process.h"
#include "blc_core.h"
#include <errno.h>
#include <unistd.h> //execve

#include <thread>

#include <vte/vte.h>
#include <sys/pipe.h>
#include <gtkmm.h>


using namespace std;
using namespace Gtk;

void TerminalSpawnAsyncCallback (VteTerminal *terminal, GPid pid, GError *error, gpointer user_data){
    
    char **argv=(char**)user_data;
    if (error){
        fprintf(stderr, "Process error: '%s'\n", error->message);
        for (auto arg=argv; *arg!=nullptr; arg++){
            fprintf(stderr, "%s ", *arg);
        }
        fprintf(stderr, "\n");
    }
    else fprintf(stderr, "Success pid %d\n", pid);
};

Process::Process(string const &directory, string const &program):directory(directory),program_name(program){
    command_path=directory+program;
    arg_vector.emplace_back(command_path.data());
};

Process::~Process(){
    for (char const *&arg: arg_vector ){
        free(&arg);
    }
};

void Process::add_arg(string const &arg){
    arg_vector.emplace_back(strdup(arg.data()));
}

void Process::run(){
    
    terminal=VTE_TERMINAL(vte_terminal_new());
    arg_vector.emplace_back(nullptr);
    
    vte_terminal_spawn_async(terminal, VTE_PTY_DEFAULT , nullptr, (char**)arg_vector.data(), nullptr, G_SPAWN_DEFAULT, nullptr, nullptr, nullptr, 1000, nullptr, TerminalSpawnAsyncCallback, arg_vector.data());
    
   
}

void Process::display_terminal(){
    
    Widget *mm_terminal=Glib::wrap(GTK_WIDGET(terminal));
          
    terminal_window=new Window();
    terminal_window->add(*mm_terminal);
    terminal_window->show_all();
}
